package class7_1_FileIO_Tourists_start;

import class7_1_FileIO_Tourists_start.mobileEntities.Car;
import class7_1_FileIO_Tourists_start.mobileEntities.MobileEntity;
import class7_1_FileIO_Tourists_start.mobileEntities.Person;
import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;

// https://stackoverflow.com/questions/27999430/javafx-clickable-line-on-canvas
public class CityMap extends Application {

    public static final int NUM_TOURISTS = 15;
    public static final int MIN_AWARDS_WIDTH = 250;
    private static Image backgroundImage = new Image(CityMap.class.getResource("img/map.png")
            .toString());
    private Pane overlay = new Pane();

    private ImageView backgroundView = new ImageView(backgroundImage);
    private Pane awardsPane;

    public static void main(String[] args) {
        launch(args);
    }

    /**
     * Each taggable must also called addTaggableToMap to add itself to the map.
     * @param node node the GUI element to add.
     */
    public void addNodeToMap(Node node) {
        overlay.getChildren().addAll(node);
    }

    public void addAwardNode(Node node) {
        awardsPane.getChildren().addAll(node);
    }

    public double getWidth(){
        return backgroundView.getImage().getWidth();
    }

    public double getHeight() {
        return backgroundView.getImage().getHeight();
    }

    @Override
    public void start(Stage primaryStage) {
        Pane root = new HBox();
        awardsPane = new VBox();
        awardsPane.setMinWidth(MIN_AWARDS_WIDTH);
        Pane mapPane = new Pane();

        backgroundView.relocate(0,0);
        mapPane.setMaxWidth(backgroundImage.getWidth());
        mapPane.setMaxHeight(backgroundImage.getHeight());

        overlay.getChildren().addAll(backgroundView);
        mapPane.getChildren().addAll(overlay);
        root.getChildren().addAll(mapPane,awardsPane);

        primaryStage.setScene(new Scene(root));
        primaryStage.show();

        addEntities();
    }

    private static MobileEntity mainCharacter = null;

    public static MobileEntity getMainCharacter() {
        if(mainCharacter == null) {
            throw new RuntimeException("getMainCharacter called before game was initialized!");
        }
        return mainCharacter;
    }

    private void addEntities() {
        // Each entity places itself on the map, so we place the goal first
        // so it will show at the bottom.
        Person.Goal goal = new Person.Goal(this);

        for(int i = 0; i < NUM_TOURISTS; i++) {
            new Car(this);
        }

        synchronized (CityMap.class) {
            if(mainCharacter == null) {
                mainCharacter = new Person(this, goal);
            } else {
                System.out.println("Warning: initializing map more than once!");
            }
        }
    }

    public static class MouseGestures {

        private double orgSceneX, orgSceneY;
        private double orgTranslateX, orgTranslateY;

        public void makeDraggable(Node node) {
            node.setOnMousePressed(circleOnMousePressedEventHandler);
            node.setOnMouseDragged(circleOnMouseDraggedEventHandler);
            node.setOnMouseReleased(circleOnMouseReleasedEventHandler);
        }

        private EventHandler<MouseEvent> circleOnMousePressedEventHandler =
                (t) -> {
                    orgSceneX = t.getSceneX();
                    orgSceneY = t.getSceneY();

                    if (t.getSource() instanceof Circle) {

                        Circle p = ((Circle) (t.getSource()));

                        orgTranslateX = p.getCenterX();
                        orgTranslateY = p.getCenterY();

//                    } else if(t.getSource() instanceof ImageView) {
//                        ImageView view = (ImageView) t.getSource();
//                        view.setImage(scaredImage);
////                        orgTranslateX = t.getX();
////                        orgTranslateY = t.getY();
//                        orgTranslateX = view.getTranslateX();
//                        orgTranslateY = view.getTranslateY();
                    } else {

                        Node p = ((Node) (t.getSource()));

                        orgTranslateX = p.getTranslateX();
                        orgTranslateY = p.getTranslateY();

                    }
                };

        private EventHandler<MouseEvent> circleOnMouseDraggedEventHandler =
            (t) -> {
                double offsetX = t.getSceneX() - orgSceneX;
                double offsetY = t.getSceneY() - orgSceneY;

                double newTranslateX = orgTranslateX + offsetX;
                double newTranslateY = orgTranslateY + offsetY;

                if (t.getSource() instanceof Circle) {

                    Circle p = ((Circle) (t.getSource()));

                    p.setCenterX(newTranslateX);
                    p.setCenterY(newTranslateY);

//                } else if( t.getSource() instanceof ImageView) {
//                    ImageView view = (ImageView) t.getSource();
//                    view.setTranslateX(t.getSceneX());
//                    view.setTranslateY(t.getSceneY());
                } else {

                    Node p = ((Node) (t.getSource()));

                    p.setTranslateX(newTranslateX);
                    p.setTranslateY(newTranslateY);
                    System.out.println("newTranslateX = " + newTranslateX);
                    System.out.println("newTranslateY = " + newTranslateY);
                }
            };

        private EventHandler<MouseEvent> circleOnMouseReleasedEventHandler = (t)-> {
//            if(t.getSource() instanceof ImageView) {
//                ImageView view = (ImageView) t.getSource();
//                view.setImage(smileyImage);
//            }
        };

    }

}