package class7_1_FileIO_Tourists_start.mobileEntities;

import class7_1_FileIO_Tourists_start.CityMap;
import javafx.geometry.Point2D;
import javafx.scene.image.Image;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;

public class Person extends MobileEntity {
    private static final Image SMILEY_IMAGE = new Image(CityMap.class.getResource("img/smiley.png")
            .toString());
    public static final int PERSON_MAX_SPEED = 2;
    private Goal goal;

    public Person(CityMap cityMap, Goal goal) {
        super(cityMap, SMILEY_IMAGE);
        setName("You");
        stepSize = new Point2D(0,0);
        this.goal = goal;
    }

    @Override
    protected void step() {
        Point2D direction = goal.getTarget().subtract(getLocation());
        if(Math.hypot(direction.getX(), direction.getY()) < PERSON_MAX_SPEED) {
            // last step to goal
            stepSize = direction;
        } else {
            // approaching goal
            stepSize = direction.multiply(PERSON_MAX_SPEED /
                    Math.hypot(direction.getX(), direction.getY()));
        }
        super.step();
    }

    /**
     * The circular shape on the map that the person approaches.
     */
    public static class Goal {

        public static final int RADIUS = 100;
        private final Circle personTarget;

        public Goal(CityMap cityMap) {
            personTarget = new Circle(RADIUS);
            personTarget.setStroke(Color.GREEN);
            personTarget.setFill(Color.GREEN.deriveColor(1, 1, 1, 0.2));
            personTarget.relocate(0, 0);

            CityMap.MouseGestures mg = new CityMap.MouseGestures();
            mg.makeDraggable(personTarget);

            cityMap.addNodeToMap(personTarget);
        }

        public Point2D getTarget() {
            return new Point2D(personTarget.getCenterX(),personTarget.getCenterY()).add(RADIUS, RADIUS);
        }
    }
}
